@@ -2,9 +2,8 @@ from __future__ import annotations
 
 import functools
 import os.path
-import typing
 from pathlib import Path
-from typing import Callable, Optional, Union
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Optional, Union
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -12,22 +11,25 @@ import pandas as pd
 import seaborn as sns
 from IPython.core.display_functions import DisplayHandle, display
 from pandas import DataFrame, Series
-from safeds.data.tabular.containers._column import Column
-from safeds.data.tabular.containers._row import Row
-from safeds.data.tabular.typing import ColumnType, TableSchema
+from safeds.data.tabular.typing import ColumnType, Schema
 from safeds.exceptions import (
     ColumnLengthMismatchError,
     ColumnSizeError,
     DuplicateColumnNameError,
     IndexOutOfBoundsError,
     MissingDataError,
-    MissingSchemaError,
     NonNumericColumnError,
     SchemaMismatchError,
     UnknownColumnNameError,
 )
 from scipy import stats
 
+from ._column import Column
+from ._row import Row
+
+if TYPE_CHECKING:
+    from ._tagged_table import TaggedTable
+
 
 # noinspection PyProtectedMember
 class Table:
@@ -38,7 +40,7 @@ class Table:
     ----------
     data : typing.Iterable
         The data.
-    schema : Optional[TableSchema]
+    schema : Optional[Schema]
         The schema of the table. If not specified, the schema will be inferred from the data.
 
     Raises
@@ -52,7 +54,7 @@ class Table:
     # ------------------------------------------------------------------------------------------------------------------
 
     @staticmethod
-    def from_csv(path: str) -> Table:
+    def from_csv_file(path: str) -> Table:
         """
         Read data from a CSV file into a table.
 
@@ -82,7 +84,7 @@ class Table:
             raise ValueError(f'Could not read file from "{path}" as CSV') from exception
 
     @staticmethod
-    def from_json(path: str) -> Table:
+    def from_json_file(path: str) -> Table:
         """
         Read data from a JSON file into a table.
 
@@ -109,9 +111,7 @@ class Table:
         except FileNotFoundError as exception:
             raise FileNotFoundError(f'File "{path}" does not exist') from exception
         except Exception as exception:
-            raise ValueError(
-                f'Could not read file from "{path}" as JSON'
-            ) from exception
+            raise ValueError(f'Could not read file from "{path}" as JSON') from exception
 
     @staticmethod
     def from_columns(columns: list[Column]) -> Table:
@@ -143,9 +143,7 @@ class Table:
         for column in columns:
             if column._data.size != columns[0]._data.size:
                 raise ColumnLengthMismatchError(
-                    "\n".join(
-                        [f"{column.name}: {column._data.size}" for column in columns]
-                    )
+                    "\n".join([f"{column.name}: {column._data.size}" for column in columns])
                 )
             dataframe[column.name] = column._data
 
@@ -176,7 +174,7 @@ class Table:
         if len(rows) == 0:
             raise MissingDataError("This function requires at least one row.")
 
-        schema_compare: TableSchema = rows[0].schema
+        schema_compare: Schema = rows[0].schema
         row_array: list[Series] = []
 
         for row in rows:
@@ -192,23 +190,17 @@ class Table:
     # Dunder methods
     # ------------------------------------------------------------------------------------------------------------------
 
-    def __init__(self, data: typing.Iterable, schema: Optional[TableSchema] = None):
-        self._data: pd.Dataframe = (
-            data if isinstance(data, pd.DataFrame) else pd.DataFrame(data)
-        )
-        if schema is None:
-            if self.count_columns() == 0:
-                raise MissingSchemaError()
-            self._schema: TableSchema = TableSchema._from_dataframe(self._data)
-        else:
-            self._schema = schema
-            if self._data.empty:
-                self._data = pd.DataFrame(columns=self._schema.get_column_names())
+    def __init__(self, data: Iterable, schema: Optional[Schema] = None):
+        self._data: pd.Dataframe = data if isinstance(data, pd.DataFrame) else pd.DataFrame(data)
+        self._schema: Schema = Schema._from_dataframe(self._data) if schema is None else schema
+
+        if self._data.empty:
+            self._data = pd.DataFrame(columns=self._schema.get_column_names())
 
         self._data = self._data.reset_index(drop=True)
         self._data.columns = list(range(self.count_columns()))
 
-    def __eq__(self, other: typing.Any) -> bool:
+    def __eq__(self, other: Any) -> bool:
         if not isinstance(other, Table):
             return NotImplemented
         if self is other:
@@ -235,13 +227,13 @@ class Table:
     # ------------------------------------------------------------------------------------------------------------------
 
     @property
-    def schema(self) -> TableSchema:
+    def schema(self) -> Schema:
         """
         Return the schema of the table.
 
         Returns
         -------
-        schema : TableSchema
+        schema : Schema
             The schema.
         """
         return self._schema
@@ -271,10 +263,8 @@ class Table:
         """
         if self._schema.has_column(column_name):
             output_column = Column(
-                self._data.iloc[
-                    :, [self._schema._get_column_index_by_name(column_name)]
-                ].squeeze(),
                 column_name,
+                self._data.iloc[:, [self._schema._get_column_index_by_name(column_name)]].squeeze(),
                 self._schema.get_type_of_column(column_name),
             )
             return output_column
@@ -533,122 +523,10 @@ class Table:
         for row in rows:
             if self._schema != row.schema:
                 raise SchemaMismatchError()
-        result = pd.concat(
-            [result, *[row._data.to_frame().T for row in rows]]
-        ).infer_objects()
+        result = pd.concat([result, *[row._data.to_frame().T for row in rows]]).infer_objects()
         result.columns = self._schema.get_column_names()
         return Table(result)
 
-    def drop_columns(self, column_names: list[str]) -> Table:
-        """
-        Return a table without the given column(s).
-
-        Parameters
-        ----------
-        column_names : list[str]
-            A list containing all columns to be dropped.
-
-        Returns
-        -------
-        table : Table
-            A table without the given columns.
-
-        Raises
-        ------
-        ColumnNameError
-            If any of the given columns do not exist.
-        """
-        invalid_columns = []
-        column_indices = []
-        for name in column_names:
-            if not self._schema.has_column(name):
-                invalid_columns.append(name)
-            else:
-                column_indices.append(self._schema._get_column_index_by_name(name))
-        if len(invalid_columns) != 0:
-            raise UnknownColumnNameError(invalid_columns)
-        transformed_data = self._data.drop(labels=column_indices, axis="columns")
-        transformed_data.columns = list(
-            name for name in self._schema.get_column_names() if name not in column_names
-        )
-        return Table(transformed_data)
-
-    def drop_columns_with_missing_values(self) -> Table:
-        """
-        Return a table without the columns that contain missing values.
-
-        Returns
-        -------
-        table : Table
-            A table without the columns that contain missing values.
-        """
-        return Table.from_columns(
-            [column for column in self.to_columns() if not column.has_missing_values()]
-        )
-
-    def drop_columns_with_non_numerical_values(self) -> Table:
-        """
-        Return a table without the columns that contain non-numerical values.
-
-        Returns
-        -------
-        table : Table
-            A table without the columns that contain non-numerical values.
-
-        """
-        return Table.from_columns(
-            [column for column in self.to_columns() if column.type.is_numeric()]
-        )
-
-    def drop_duplicate_rows(self) -> Table:
-        """
-        Return a copy of the table with every duplicate row removed.
-
-        Returns
-        -------
-        result : Table
-            The table with the duplicate rows removed.
-        """
-        df = self._data.drop_duplicates(ignore_index=True)
-        df.columns = self._schema.get_column_names()
-        return Table(df)
-
-    def drop_rows_with_missing_values(self) -> Table:
-        """
-        Return a table without the rows that contain missing values.
-
-        Returns
-        -------
-        table : Table
-            A table without the rows that contain missing values.
-        """
-        result = self._data.copy(deep=True)
-        result = result.dropna(axis="index")
-        return Table(result, self._schema)
-
-    def drop_rows_with_outliers(self) -> Table:
-        """
-        Remove all rows from the table that contain at least one outlier.
-
-        We define an outlier as a value that has a distance of more than 3 standard deviations from the column mean.
-        Missing values are not considered outliers. They are also ignored during the calculation of the standard
-        deviation.
-
-        Returns
-        -------
-        new_table : Table
-            A new table without rows containing outliers.
-        """
-        copy = self._data.copy(deep=True)
-
-        table_without_nonnumericals = self.drop_columns_with_non_numerical_values()
-        z_scores = np.absolute(
-            stats.zscore(table_without_nonnumericals._data, nan_policy="omit")
-        )
-        filter_ = ((z_scores < 3) | np.isnan(z_scores)).all(axis=1)
-
-        return Table(copy[filter_], self._schema)
-
     def filter_rows(self, query: Callable[[Row], bool]) -> Table:
         """
         Return a table with rows filtered by Callable (e.g. lambda function).
@@ -699,11 +577,111 @@ class Table:
         if len(invalid_columns) != 0:
             raise UnknownColumnNameError(invalid_columns)
         transformed_data = self._data[column_indices]
-        transformed_data.columns = list(
-            name for name in self._schema.get_column_names() if name in column_names
-        )
+        transformed_data.columns = list(name for name in self._schema.get_column_names() if name in column_names)
         return Table(transformed_data)
 
+    def remove_columns(self, column_names: list[str]) -> Table:
+        """
+        Return a table without the given column(s).
+
+        Parameters
+        ----------
+        column_names : list[str]
+            A list containing all columns to be dropped.
+
+        Returns
+        -------
+        table : Table
+            A table without the given columns.
+
+        Raises
+        ------
+        ColumnNameError
+            If any of the given columns do not exist.
+        """
+        invalid_columns = []
+        column_indices = []
+        for name in column_names:
+            if not self._schema.has_column(name):
+                invalid_columns.append(name)
+            else:
+                column_indices.append(self._schema._get_column_index_by_name(name))
+        if len(invalid_columns) != 0:
+            raise UnknownColumnNameError(invalid_columns)
+        transformed_data = self._data.drop(labels=column_indices, axis="columns")
+        transformed_data.columns = list(name for name in self._schema.get_column_names() if name not in column_names)
+        return Table(transformed_data)
+
+    def remove_columns_with_missing_values(self) -> Table:
+        """
+        Return a table without the columns that contain missing values.
+
+        Returns
+        -------
+        table : Table
+            A table without the columns that contain missing values.
+        """
+        return Table.from_columns([column for column in self.to_columns() if not column.has_missing_values()])
+
+    def remove_columns_with_non_numerical_values(self) -> Table:
+        """
+        Return a table without the columns that contain non-numerical values.
+
+        Returns
+        -------
+        table : Table
+            A table without the columns that contain non-numerical values.
+
+        """
+        return Table.from_columns([column for column in self.to_columns() if column.type.is_numeric()])
+
+    def remove_duplicate_rows(self) -> Table:
+        """
+        Return a copy of the table with every duplicate row removed.
+
+        Returns
+        -------
+        result : Table
+            The table with the duplicate rows removed.
+        """
+        df = self._data.drop_duplicates(ignore_index=True)
+        df.columns = self._schema.get_column_names()
+        return Table(df)
+
+    def remove_rows_with_missing_values(self) -> Table:
+        """
+        Return a table without the rows that contain missing values.
+
+        Returns
+        -------
+        table : Table
+            A table without the rows that contain missing values.
+        """
+        result = self._data.copy(deep=True)
+        result = result.dropna(axis="index")
+        return Table(result, self._schema)
+
+    def remove_rows_with_outliers(self) -> Table:
+        """
+        Remove all rows from the table that contain at least one outlier.
+
+        We define an outlier as a value that has a distance of more than 3 standard deviations from the column mean.
+        Missing values are not considered outliers. They are also ignored during the calculation of the standard
+        deviation.
+
+        Returns
+        -------
+        new_table : Table
+            A new table without rows containing outliers.
+        """
+        copy = self._data.copy(deep=True)
+
+        table_without_nonnumericals = self.remove_columns_with_non_numerical_values()
+        z_scores = np.absolute(stats.zscore(table_without_nonnumericals._data, nan_policy="omit"))
+        filter_ = ((z_scores < 3) | np.isnan(z_scores)).all(axis=1)
+
+        return Table(copy[filter_], self._schema)
+
     def rename_column(self, old_name: str, new_name: str) -> Table:
         """
         Rename a single column.
@@ -769,10 +747,7 @@ class Table:
         if old_column_name not in self._schema.get_column_names():
             raise UnknownColumnNameError([old_column_name])
 
-        if (
-            new_column.name in self._schema.get_column_names()
-            and new_column.name != old_column_name
-        ):
+        if new_column.name in self._schema.get_column_names() and new_column.name != old_column_name:
             raise DuplicateColumnNameError(new_column.name)
 
         if self.count_rows() != new_column._data.size:
@@ -789,7 +764,7 @@ class Table:
         result[new_column.name] = new_column._data
         return Table(result)
 
-    def shuffle(self) -> Table:
+    def shuffle_rows(self) -> Table:
         """
         Shuffle the table randomly.
 
@@ -803,10 +778,10 @@ class Table:
         new_df.columns = self._schema.get_column_names()
         return Table(new_df)
 
-    def slice(
+    def slice_rows(
         self,
-        start: typing.Optional[int] = None,
-        end: typing.Optional[int] = None,
+        start: Optional[int] = None,
+        end: Optional[int] = None,
         step: int = 1,
     ) -> Table:
         """
@@ -838,13 +813,7 @@ class Table:
         if end is None:
             end = self.count_rows()
 
-        if (
-            start < 0
-            or end < 0
-            or start >= self.count_rows()
-            or end > self.count_rows()
-            or end < start
-        ):
+        if start < 0 or end < 0 or start >= self.count_rows() or end > self.count_rows() or end < start:
             raise ValueError("the given index is out of bounds")
 
         new_df = self._data.iloc[start:end:step]
@@ -853,24 +822,25 @@ class Table:
 
     def sort_columns(
         self,
-        query: Callable[[Column, Column], int] = lambda col1, col2: (
-            col1.name > col2.name
-        )
+        comparator: Callable[[Column, Column], int] = lambda col1, col2: (col1.name > col2.name)
         - (col1.name < col2.name),
     ) -> Table:
         """
-        Sort a table with the given lambda function.
-        If no function is given the columns will be sorted alphabetically.
-        This function uses the default python sort algorithm.
-        The query returns
-        0, if both columns are equal.
-        < 0, if the first column should be ordered after the second column.
-        > 0, if the first column should be ordered before the second column.
+        Sort the columns of a `Table` with the given comparator and return a new `Table`. The original table is not
+        modified.
+
+        The comparator is a function that takes two columns `col1` and `col2` and returns an integer:
+
+        * If `col1` should be ordered before `col2`, the function should return a negative number.
+        * If `col1` should be ordered after `col2`, the function should return a positive number.
+        * If the original order of `col1` and `col2` should be kept, the function should return 0.
+
+        If no comparator is given, the columns will be sorted alphabetically by their name.
 
         Parameters
         ----------
-        query : a lambda function
-            The lambda function used to sort the columns.
+        comparator : Callable[[Column, Column], int]
+            The function used to compare two columns.
 
         Returns
         -------
@@ -878,10 +848,35 @@ class Table:
             A new table with sorted columns.
         """
         columns = self.to_columns()
-        columns.sort(key=functools.cmp_to_key(query))
+        columns.sort(key=functools.cmp_to_key(comparator))
         return Table.from_columns(columns)
 
-    def split(self, percentage_in_first: float) -> typing.Tuple[Table, Table]:
+    def sort_rows(self, comparator: Callable[[Row, Row], int]) -> Table:
+        """
+        Sort the rows of a `Table` with the given comparator and return a new `Table`. The original table is not
+        modified.
+
+        The comparator is a function that takes two rows `row1` and `row2` and returns an integer:
+
+        * If `row1` should be ordered before `row2`, the function should return a negative number.
+        * If `row1` should be ordered after `row2`, the function should return a positive number.
+        * If the original order of `row1` and `row2` should be kept, the function should return 0.
+
+        Parameters
+        ----------
+        comparator : Callable[[Row, Row], int]
+            The function used to compare two rows.
+
+        Returns
+        -------
+        new_table : Table
+            A new table with sorted rows.
+        """
+        rows = self.to_rows()
+        rows.sort(key=functools.cmp_to_key(comparator))
+        return Table.from_rows(rows)
+
+    def split(self, percentage_in_first: float) -> tuple[Table, Table]:
         """
         Split the table into two new tables.
 
@@ -901,13 +896,32 @@ class Table:
         if percentage_in_first <= 0 or percentage_in_first >= 1:
             raise ValueError("the given percentage is not in range")
         return (
-            self.slice(0, round(percentage_in_first * self.count_rows())),
-            self.slice(round(percentage_in_first * self.count_rows())),
+            self.slice_rows(0, round(percentage_in_first * self.count_rows())),
+            self.slice_rows(round(percentage_in_first * self.count_rows())),
         )
 
-    def transform_column(
-        self, name: str, transformer: Callable[[Row], typing.Any]
-    ) -> Table:
+    def tag_columns(self, target_name: str, feature_names: Optional[list[str]] = None) -> TaggedTable:
+        """
+        Mark the columns of the table as target column or feature columns. The original table is not modified.
+
+        Parameters
+        ----------
+        target_name : str
+            Name of the target column.
+        feature_names : Optional[list[str]]
+            Names of the feature columns. If None, all columns except the target column are used.
+
+        Returns
+        -------
+        tagged_table : TaggedTable
+            A new tagged table with the given target and feature names.
+        """
+        # pylint: disable=import-outside-toplevel
+        from ._tagged_table import TaggedTable
+
+        return TaggedTable(self._data, target_name, feature_names, self._schema)
+
+    def transform_column(self, name: str, transformer: Callable[[Row], Any]) -> Table:
         """
         Transform provided column by calling provided transformer.
 
@@ -924,7 +938,7 @@ class Table:
         """
         if self.has_column(name):
             items: list = [transformer(item) for item in self.to_rows()]
-            result: Column = Column(pd.Series(items), name)
+            result: Column = Column(name, pd.Series(items))
             return self.replace_column(name, result)
         raise UnknownColumnNameError([name])
 
@@ -936,7 +950,7 @@ class Table:
         """
         Plot a correlation heatmap for all numerical columns of this `Table`.
         """
-        only_numerical = self.drop_columns_with_non_numerical_values()
+        only_numerical = self.remove_columns_with_non_numerical_values()
 
         sns.heatmap(
             data=only_numerical._data.corr(),
@@ -1023,7 +1037,7 @@ class Table:
     # Conversion
     # ------------------------------------------------------------------------------------------------------------------
 
-    def to_csv(self, path_to_file: str) -> None:
+    def to_csv_file(self, path: str) -> None:
         """
         Write the data from the table into a CSV file.
         If the file and/or the directories do not exist they will be created.
@@ -1031,15 +1045,15 @@ class Table:
 
         Parameters
         ----------
-        path_to_file : str
+        path : str
             The path to the output file.
         """
-        Path(os.path.dirname(path_to_file)).mkdir(parents=True, exist_ok=True)
+        Path(os.path.dirname(path)).mkdir(parents=True, exist_ok=True)
         data_to_csv = self._data.copy()
         data_to_csv.columns = self._schema.get_column_names()
-        data_to_csv.to_csv(path_to_file, index=False)
+        data_to_csv.to_csv(path, index=False)
 
-    def to_json(self, path_to_file: str) -> None:
+    def to_json_file(self, path: str) -> None:
         """
         Write the data from the table into a JSON file.
         If the file and/or the directories do not exist, they will be created.
@@ -1047,13 +1061,13 @@ class Table:
 
         Parameters
         ----------
-        path_to_file : str
+        path : str
             The path to the output file.
         """
-        Path(os.path.dirname(path_to_file)).mkdir(parents=True, exist_ok=True)
+        Path(os.path.dirname(path)).mkdir(parents=True, exist_ok=True)
         data_to_json = self._data.copy()
         data_to_json.columns = self._schema.get_column_names()
-        data_to_json.to_json(path_to_file)
+        data_to_json.to_json(path)
 
     def to_columns(self) -> list[Column]:
         """
@@ -1075,9 +1089,7 @@ class Table:
         rows : list[Row]
             List of rows.
         """
-        return [
-            Row(series_row, self._schema) for (_, series_row) in self._data.iterrows()
-        ]
+        return [Row(series_row, self._schema) for (_, series_row) in self._data.iterrows()]
 
     # ------------------------------------------------------------------------------------------------------------------
     # Other
@@ -1095,7 +1107,5 @@ class Table:
         tmp = self._data.copy(deep=True)
         tmp.columns = self.get_column_names()
 
-        with pd.option_context(
-            "display.max_rows", tmp.shape[0], "display.max_columns", tmp.shape[1]
-        ):
+        with pd.option_context("display.max_rows", tmp.shape[0], "display.max_columns", tmp.shape[1]):
             return display(tmp)
