@@ -1,4 +1,3 @@
-from ._imputer import Imputer
+from ._imputer import Imputer, ImputerStrategy
 from ._label_encoder import LabelEncoder
 from ._one_hot_encoder import OneHotEncoder
-from ._ordinal_encoder import OrdinalEncoder
