@@ -17,6 +17,7 @@ class GradientBoosting(Classifier):
 
     def __init__(self) -> None:
         self._wrapped_classifier: Optional[sk_GradientBoostingClassifier] = None
+        self._feature_names: Optional[list[str]] = None
         self._target_name: Optional[str] = None
 
     def fit(self, training_set: TaggedTable) -> GradientBoosting:
@@ -45,6 +46,7 @@ class GradientBoosting(Classifier):
 
         result = GradientBoosting()
         result._wrapped_classifier = wrapped_classifier
+        result._feature_names = training_set.features.get_column_names()
         result._target_name = training_set.target.name
 
         return result
@@ -69,4 +71,15 @@ class GradientBoosting(Classifier):
         PredictionError
             If prediction with the given dataset failed.
         """
-        return predict(self._wrapped_classifier, dataset, self._target_name)
+        return predict(self._wrapped_classifier, dataset, self._feature_names, self._target_name)
+
+    def is_fitted(self) -> bool:
+        """
+        Checks if the classifier is fitted.
+
+        Returns
+        -------
+        is_fitted : bool
+            Whether the classifier is fitted.
+        """
+        return self._wrapped_classifier is not None
