@@ -16,6 +16,7 @@ class RandomForest(Classifier):
 
     def __init__(self) -> None:
         self._wrapped_classifier: Optional[sk_RandomForestClassifier] = None
+        self._feature_names: Optional[list[str]] = None
         self._target_name: Optional[str] = None
 
     def fit(self, training_set: TaggedTable) -> RandomForest:
@@ -44,6 +45,7 @@ class RandomForest(Classifier):
 
         result = RandomForest()
         result._wrapped_classifier = wrapped_classifier
+        result._feature_names = training_set.features.get_column_names()
         result._target_name = training_set.target.name
 
         return result
@@ -67,4 +69,15 @@ class RandomForest(Classifier):
         PredictionError
             If prediction with the given dataset failed.
         """
-        return predict(self._wrapped_classifier, dataset, self._target_name)
+        return predict(self._wrapped_classifier, dataset, self._feature_names, self._target_name)
+
+    def is_fitted(self) -> bool:
+        """
+        Checks if the classifier is fitted.
+
+        Returns
+        -------
+        is_fitted : bool
+            Whether the classifier is fitted.
+        """
+        return self._wrapped_classifier is not None
