@@ -56,6 +56,17 @@ class Regressor(ABC):
             If prediction with the given dataset failed.
         """
 
+    @abstractmethod
+    def is_fitted(self) -> bool:
+        """
+        Checks if the regressor is fitted.
+
+        Returns
+        -------
+        is_fitted : bool
+            Whether the regressor is fitted.
+        """
+
     # noinspection PyProtectedMember
     def mean_squared_error(self, validation_or_test_set: TaggedTable) -> float:
         """
@@ -110,7 +121,5 @@ def _check_metrics_preconditions(actual: Column, expected: Column) -> None:
 
     if actual._data.size != expected._data.size:
         raise ColumnLengthMismatchError(
-            "\n".join(
-                [f"{column.name}: {column._data.size}" for column in [actual, expected]]
-            )
+            "\n".join([f"{column.name}: {column._data.size}" for column in [actual, expected]])
         )
