@@ -17,6 +17,7 @@ class RidgeRegression(Regressor):
 
     def __init__(self) -> None:
         self._wrapped_regressor: Optional[sk_Ridge] = None
+        self._feature_names: Optional[list[str]] = None
         self._target_name: Optional[str] = None
 
     def fit(self, training_set: TaggedTable) -> RidgeRegression:
@@ -45,6 +46,7 @@ class RidgeRegression(Regressor):
 
         result = RidgeRegression()
         result._wrapped_regressor = wrapped_regressor
+        result._feature_names = training_set.features.get_column_names()
         result._target_name = training_set.target.name
 
         return result
@@ -68,4 +70,15 @@ class RidgeRegression(Regressor):
         PredictionError
             If prediction with the given dataset failed.
         """
-        return predict(self._wrapped_regressor, dataset, self._target_name)
+        return predict(self._wrapped_regressor, dataset, self._feature_names, self._target_name)
+
+    def is_fitted(self) -> bool:
+        """
+        Check if the regressor is fitted.
+
+        Returns
+        -------
+        is_fitted : bool
+            Whether the regressor is fitted.
+        """
+        return self._wrapped_regressor is not None
